-------------------------------------------------------------------------------
-- Title      : Big  64 bits  Timer 
-- Project    : 
-------------------------------------------------------------------------------
-- File       : timer64bits.vhd
-- Author     :   <patrice@SOLEIL>
-- Company    : 
-- Created    : 2010-01-10
-- Last update: 2010/01/24
-- Platform   : 
-- Standard   : VHDL'93
-------------------------------------------------------------------------------
-- Description: Big 64 bits Timer always running with 32 bits interface
--            : Control register - write only - to program capture  - address 
--            : High 32 bits data register - read only - address 0
--            : Low 32 bits data register - read only - addresse 1
-------------------------------------------------------------------------------
-- Copyright (c) 2010
-- http://vhdl33.free.fr
-------------------------------------------------------------------------------
-- Revisions  :
-- Date        Version  Author  Description
-- 2010-01-10  1.0      patrice Created
-------------------------------------------------------------------------------

LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.numeric_std.ALL;

ENTITY timer64bits IS
  PORT (
    reset    : IN  std_ulogic;          -- clear Timer
    clk      : IN  std_ulogic;          -- increment
    addr     : IN  std_ulogic;          -- Two adresses
    cs       : IN  std_ulogic;          -- Chip select tristate bus control
    rd, wr   : IN  std_ulogic;          -- transfer direction
    data_in  : IN  std_logic_vector(31 DOWNTO 0);  -- For writing control register
    data_out : OUT std_logic_vector(31 DOWNTO 0));  -- 2 reading for complete data
END timer64bits;


ARCHITECTURE complete OF timer64bits IS

  SIGNAL counter64        : unsigned(63 DOWNTO 0):= X"00000000FFFFFFFE";          -- timer heart
  SIGNAL control_register : std_logic_vector(1 DOWNTO 0);   -- 2 bits only
  SIGNAL capture_register : std_logic_vector(63 DOWNTO 0);  -- sample timer

BEGIN  -- complete


  -- purpose: always runing
  core : PROCESS
  BEGIN  -- PROCESS core
    WAIT UNTIL rising_edge(clk);        -- synchronous
    IF control_register(1) = '1' THEN   -- see control register
      counter64 <= (OTHERS => '0');
    ELSE
      counter64 <= counter64 + "01";
    END IF;
  END PROCESS core;

  -- purpose: 32 bits Control register (write only)
  -- Bit 0 : capture timer value, transfer in output register
  -- Bit 1 : clear the timer
  -- no address defined for this register
  ctrl : PROCESS
  BEGIN  -- PROCESS ctrl
    WAIT UNTIL rising_edge(clk);        -- synchronous
    IF reset = '1' THEN
      control_register   <= "11";       -- reset clear timer and load capture register
    ELSE
      control_register   <= "00";       -- clearing after one period
      IF cs = '1' AND wr = '1' THEN
        control_register <= data_in(1 DOWNTO 0);
      END IF;
    END IF;
  END PROCESS ctrl;

  -- purpose: 64 bits Capture REGISTER
  captr : PROCESS
  BEGIN  -- PROCESS captr
    WAIT UNTIL rising_edge(clk);        -- synchronous
    IF control_register(0) = '1' THEN
      capture_register <= std_logic_vector(counter64);
    END IF;
  END PROCESS captr;

  -- multiplexed output ( 2X 32 bits)
  data_out <= capture_register(63 DOWNTO 32)     WHEN (addr = '0' AND cs = '1' AND rd = '1')
              ELSE capture_register(31 DOWNTO 0) WHEN (addr = '1' AND cs = '1' AND rd = '1')
              ELSE (OTHERS => 'Z');

END complete;
